<?php

class DPD_UK_Shipping extends WPRuby_Shipping_Method
{

    /**
     * @var mixed
     */
    private $weight;

    /**
     * @var mixed
     */
    private $delivery_options;

    /** @var array */
    private $default_size;

    /**
     * @var string
     */
    private $courier_collection;

    /**
     * @var boolean
     */
    private $add_signature;

    /** @var string */
    private $intl_delivery_enabled;

    /**
     * @var boolean
     */
    private $add_cover;

    /**
     * Constructor for your shipping class
     * @access public
     *
     * @param int $instance_id
     */
    public function __construct($instance_id = 0)
    {

        $this->id = WPRUBY_DPD_UK_PLUGIN_ID;
        $this->method_title = __('DPD UK', "wc-royal-mail");
        $this->method_description = __("A DPD UK shipping calculator", "wc-royal-mail");
        $this->instance_id = absint($instance_id);
        $this->supports  = ['shipping-zones', 'instance-settings'];
        $this->init();
    }

    /**
     * Init your settings
     *
     * @access public
     * @return void
     */
    public function init()
    {
        // Load the settings API
        $this->init_form_fields();
        $this->init_settings();

        $this->enabled = $this->settings['enabled'] ?? 'yes';
        $this->title = $this->settings['title'] ?? __( 'DPD UK', "wc-royal-mail" );


        $this->debug_mode = $this->get_option('debug_mode');
        $this->weight = $this->get_option('weight');
        $this->delivery_options = $this->get_option('delivery_options');
        $this->courier_collection = $this->get_option('courier_collection');
        $this->add_signature = $this->get_option('add_signature');
        $this->add_cover = $this->get_option('add_cover');
        $this->intl_delivery_enabled = $this->get_option('intl_delivery_enabled');
	    $this->default_size = $this->get_option('default_size');
        add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
    }



    /**
     * Define settings field for this shipping
     * @return void
     */
    public function init_form_fields()
    {

        $this->instance_form_fields = [
            'enabled' => [
                'title' => __('Enable', "wc-royal-mail"),
                'type' => 'checkbox',
                'description' => __('Enable DPD UK', "wc-royal-mail"),
                'default' => 'yes'
            ],

            'title' => [
                'title' => __('Title', "wc-royal-mail"),
                'type' => 'text',
                'description' => __('Title to be display in the Cart & Checkout pages.', "wc-royal-mail"),
                'default' => __('DPD', "wc-royal-mail")
            ],

            'debug_mode' => [
                'title' => __('Enable Debug Mode', "wc-royal-mail"),
                'type' => 'checkbox',
                'label' => __('Enable', 'wc-royal-mail'),
                'description' => __('If debug mode is enabled, the shipping method will be activated just for the administrator. The debug mode will display all the debugging data at the cart and the checkout pages.'),
                'default' => 'no'
            ],

            'weight' => [
                'title' => __('Default Package Weight (g)', "wc-royal-mail"),
                'type' => 'number',
                'description' => __('Default DPD package weight to be use (in case product weight is not specified)', "wc-royal-mail"),
                'default' => 1000
            ],
            'default_size' => [
	            'type' => 'default_size',
            ],
            'domestic_options' => [
	            'title' => __('Domestic Options', 'wc-royal-mail'),
	            'type' => 'multiselect',
	            'description' => __('Select the delivery option.', 'wc-royal-mail'),
	            'class'     => 'availability wc-enhanced-select',
	            'css'         => 'width:80%;',
	            'options' => [
		            'dpd-dom-s2s-nd' => __('Shop to Shop', 'wc-royal-mail'),
		            'dpd-dom-ds-nd' => __('Drop off at Shop', 'wc-royal-mail'),
		            'dpd-dom-d2d-nd-ep' => __('Door 2 Door - Next Day', 'wc-royal-mail'),
		            'dpd-dom-d2d-nd-12-ep' => __('Door 2 Door - By 12', 'wc-royal-mail'),
		            'dpd-dom-d2d-nd-10-ep' => __('Door 2 Door - By 10:30', 'wc-royal-mail'),
	            ],
	            'default' => ['dpd-dom-s2s-nd', 'dpd-dom-ds-nd', 'dpd-dom-d2d-nd-ep'],
            ],

            'intl_options' => [
	            'title' => __('International Options', 'wc-royal-mail'),
	            'type' => 'multiselect',
	            'description' => __('Select the delivery option.', 'wc-royal-mail'),
	            'class'     => 'availability wc-enhanced-select',
	            'css'         => 'width:80%;',
	            'options' => [
		            'dpd-int-ds' => __('Drop off at Shop - DPD Classic', 'wc-royal-mail'),
		            'dpd-int-d2d' => __('Door 2 Door - DPD Classic', 'wc-royal-mail'),
		            'dpd-int-ds-ac' => __('Drop off at Shop - Air Classic', 'wc-royal-mail'),
		            'dpd-int-d2d-ac' => __('Door 2 Door - Air Classic', 'wc-royal-mail'),
		            'dpd-int-ds-ae' => __('Drop off at Shop - Air Express', 'wc-royal-mail'),
		            'dpd-int-d2d-ae' => __('Door 2 Door - Air Express', 'wc-royal-mail'),
	            ],
	            'default' => ['dpd-int-ds', 'dpd-int-ds-ac', 'dpd-int-ds-ae'],
            ],
            'add_cover' => [
                'title' => __('Add Cover', 'wc-royal-mail'),
                'type' => 'checkbox',
                'description' => __('Add item insurance fees to the shipping cost.', 'wc-royal-mail'),
                'label' => __('Enable ', 'wc-royal-mail'),
                'default' => 'no',
            ],
        ];
    }

    /**
     * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
     *
     * @access public
     * @param mixed $package
     * @return void
     */
    public function calculate_shipping($package = array())
    {
	    $calculator = new DPD_Calculator($this->instance_settings);
	    $rates = $calculator->calculate($package);
	    foreach ($rates as $rate) {
		    $this->add_rate($rate);
	    }

    }

	/**
	 * @return string
	 */
	public function generate_default_size_html()
	{
		$length = (isset($this->instance_settings['default_size']['length']))?$this->instance_settings['default_size']['length']:'';
		$width = (isset($this->instance_settings['default_size']['width']))?$this->instance_settings['default_size']['width']:'';
		$height = (isset($this->instance_settings['default_size']['height']))?$this->instance_settings['default_size']['height']:'';
		ob_start();
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="woocommerce_wpruby_royalmail_debug_mode"><?php _e('Default Package Size', 'wc-royal-mail') ?></label>
			</th>
			<td class="forminp">
				<fieldset>
					<label for="woocommerce_wpruby_royalmail_default_length"><?php _e('Length (mm)', 'wc-royal-mail'); ?></label> <input id="woocommerce_wpruby_royalmail_default_length" name="woocommerce_wpruby_royalmail_default_length" type="text" value="<?php echo esc_attr($length); ?>" style="width:70px" />
					<label for="woocommerce_wpruby_royalmail_default_width"><?php _e('Width (mm)', 'wc-royal-mail'); ?></label>  <input id="woocommerce_wpruby_royalmail_default_width" name="woocommerce_wpruby_royalmail_default_width" type="text" value="<?php echo esc_attr($width); ?>" style="width:70px" />
					<label for="woocommerce_wpruby_royalmail_default_height"><?php _e('Height (mm)', 'wc-royal-mail'); ?></label> <input id="woocommerce_wpruby_royalmail_default_height" name="woocommerce_wpruby_royalmail_default_height" type="text" value="<?php echo esc_attr($height); ?>" style="width:70px" />
				</fieldset>
			</td>
		</tr>

		<?php
		return ob_get_clean();

	}

	/**
	 * validate_default_size_field function.
	 *
	 * @access public
	 *
	 * @param mixed $key
	 *
	 * @return array
	 */
	public function validate_default_size_field($key)
	{
		$dimensions = [];
		if(is_numeric($_POST['woocommerce_wpruby_royalmail_default_length']) && $_POST['woocommerce_wpruby_royalmail_default_length'] > 0)
			$dimensions['length'] = $_POST['woocommerce_wpruby_royalmail_default_length'];
		if(is_numeric($_POST['woocommerce_wpruby_royalmail_default_width']) && $_POST['woocommerce_wpruby_royalmail_default_width'] > 0)
			$dimensions['width']  = $_POST['woocommerce_wpruby_royalmail_default_width'];
		if(is_numeric($_POST['woocommerce_wpruby_royalmail_default_height']) && $_POST['woocommerce_wpruby_royalmail_default_height'] > 0)
			$dimensions['height'] = $_POST['woocommerce_wpruby_royalmail_default_height'];

		return $dimensions;
	}

}
