<?php


class WPRuby_Evri_Intl_Calculator extends WPRuby_Abstract_Calculator
{
	private $settings;

	public function __construct($settings)
	{
		$this->settings = $settings;
	}


	/**
	 * @return array
	 */
	public function calculate($cart)
	{

		$packages = $this->get_package_details_by_boxpacker($cart);

		if (!is_array($packages)) {
			return [];
		}

		return [ $this->get_price_for_packages($packages, $cart) ];

	}

	private function get_price_for_packages($packages, $cart)
	{

		$parcels = [];
		foreach ($packages as $package) {
			$parcels[] = [
				"Weight" => $package['weight'],
				"Length" => $package['length'],
				"Width" =>  $package['width'],
				"Height" => $package['height'],
			];
		}

		$countries = include ROYALMAILPRO_DIR . 'shared/data/countries.php';
		$countryName = WC()->countries->get_countries()[$cart['destination']['country']];
		$normalizedCountryName = $this->normalizeCountryName($countryName);

		if (!isset($countries[$normalizedCountryName])) {
			return [];
		}

		$body = [
			'CollectionCountry' => 219, // UK
			'CollectionPostcode' => '',
			'CollectionTown' => '',
			'DeliveryCountry' => intval($countries[$normalizedCountryName]),
			'DeliveryPostcode' => $cart['destination']['postcode'],
			'DeliveryTown' => $cart['destination']['city'],
			'Parcels' => $parcels,
			"QuoteType" => "",
			"MultiDeliveryDestinations" => false,
			"IsBusinessRecipient" => false,
			"QuoteModified" => false,
			"QuoteRequestType" => "Default",
			"ConfigurationProfile" => "EVRI.UK.LIVE"

		];

		$args = [
			'headers' => ['Content-Type' => 'application/json'],
			'body' => json_encode($body),
			'method' => 'POST',
		];

		$response = wp_remote_post('https://international.evri.com/quotes/api/results/all', $args);

		$responseBody = json_decode(wp_remote_retrieve_body($response), true);

		if (!( isset($responseBody['Quotes']) && is_array($responseBody['Quotes']) )) {
			return [];
		}

		foreach ($responseBody['Quotes'] as $quote) {
			$serviceSlug = $quote['Service']['ServiceSlug'];
			if ($this->settings['courier_collection'] === 'yes' && $this->isCollection($serviceSlug)) {
				return  [
					'id'    => $serviceSlug,
					'label' => $this->settings['title'],
					'cost'  => $quote['Total'],
				];
			}
			if ($this->settings['courier_collection'] === 'no' && $this->isDropOff($serviceSlug)) {
				return [
					'id'    => $serviceSlug,
					'label' => $this->settings['title'],
					'cost'  => $quote['Total'],
				];
			}
		}

		return [];

	}

	/**
	 * @param string $serviceSlug
	 *
	 * @return bool
	 */
	private function isDropOff($serviceSlug)
	{
		$serviceSlug = strtolower($serviceSlug);
		if (strpos($serviceSlug, 'dropoff') !== false) {
			return true;
		}

		if (strpos($serviceSlug, 'drop-off') !== false) {
			return true;
		}

		return false;
	}

	/**
	 * @param string $serviceSlug
	 *
	 * @return bool
	 */
	private function isCollection($serviceSlug)
	{
		return (strpos(strtolower($serviceSlug), 'collection') !== false);
	}

}
